// Mobile Router
// =============

// Includes file dependencies
define([
    "jquery", 
    "backbone",
    "utils",
    "configs",
    "common/MenuView",
    "common/PopupView",
    "seismic/SeismicListView",
    "seismic/collections/SeismicCollection",
    "reports/ReportListView",
    "reports/ReportDetailView",
    "reports/collections/ReportCollection",
    "news/NewsListView",
    "news/collections/NewsCollection",
    "static/StaticView",
], function($, Backbone, Utils, Configs, MenuView, PopupView, 
            SeismicListView, SeismicCollection, ReportListView, ReportDetailView, 
            ReportCollection, NewsListView, NewsCollection, StaticView) {

    // Extends Backbone.Router
    var MobileRouter = Backbone.Router.extend({

        // Backbone.js Routes
        routes: {
            // When there is no hash bang on the url, the home method is called
            "":             "homePage",
            "seismic":      "seismicListPage",
            "reports":      "reportsListPage",
            "reports/:cid": "reportsPage",
            "news":         "newsPage",
            "steps":        "stepsStaticPage",
            "disclaimer":   "disclaimerStaticPage",
            "back":         "backButton",
        },

        // The Router constructor
        initialize: function () 
        {
            var self = this;
            this.exitOnBack = false;
            this.popup = null;
            
            document.addEventListener("menubutton", function() {
                if ($( ".ui-page-active" ).jqmData( "panel" ) !== "open")
                    $( "#panelMenu" ).panel("open");
                else
                    $( "#panelMenu" ).panel("close");
            }, false);

            document.addEventListener("backbutton", function(e){
                if(self.exitOnBack) {
                    e.preventDefault();
                    navigator.app.exitApp();
                }
                else
                    Backbone.history.navigate("back", {trigger: true});
            }, false);

            // Tells Backbone to start watching for hashchange events
            Backbone.history.start();
        },

        // Home method
        homePage: function ()
        {
            // if i have a popup opened, i close it
            if (this.popup) {
                this.popup.closePopup();
                this.popup = null;
            }

            Backbone.history.navigate("seismic", {trigger: true});
        },

        // Seismic list method
        seismicListPage: function () 
        {
            this.exitOnBack = true;

            $.mobile.loading("show", {
                text: "A carregar informação sismológica...",
                textVisible: true,
                theme: "b"
            });

            var self = this;

            var collection = self.getCache('seismic', new SeismicCollection());
            var page = new SeismicListView({collection: collection});
            var serveCache = !Utils.isOnline();

            collection.fetch({
                cache: true, 
                expires: Configs.cacheExpires.seismic, 
                serveCache: serveCache,
            }).done(function () {
                self.changePage( page, {'prependPageContent': new MenuView()} );
            }).fail(function () {

                self.changePage( page, {'prependPageContent': new MenuView()} );
                
                if (!self.popup) 
                    self.popup = new PopupView({
                        'title': "Sem ligação", 
                        'content': "Não foi possível obter informação Sísmica.</br> Verifique se possui uma ligação à rede activa. <a href='#' class='ui-btn ui-mini'>Tentar novamente</a>"
                    }).render();

                self.popup.openPopup();
            });
        },

        // Report list method
        reportsListPage: function () 
        {
            this.exitOnBack = false;

            $.mobile.loading("show", {
                text: "A carregar comunicados...",
                textVisible: true,
                theme: "b"
            });

            var self = this;

            var collection = self.getCache('reports', new ReportCollection());
            var page = new ReportListView({collection: collection});
            var serveCache = !Utils.isOnline();

            collection.fetch({
                cache: true, 
                expires: Configs.cacheExpires.reports, 
                serveCache: serveCache,
            }).done(function () {
                self.changePage( page, {'prependPageContent': new MenuView()} );
            }).fail(function () {

                self.changePage( page, {'prependPageContent': new MenuView()} );
                
                if (!self.popup)  
                    self.popup = new PopupView({
                        'title': "Sem ligação", 
                        'content': "Não foi possível obter Comunicados.</br> Verifique se possui uma ligação à rede activa. <a href='#' class='ui-btn ui-mini'>Voltar</a>"
                    }).render();

                self.popup.openPopup();
            });
        },

        // Report method
        reportsPage: function (cid) 
        {
            this.exitOnBack = false;
            this.changePage( new ReportDetailView({model: this.getCache('reports').get(cid)}), false );
        },

        // News method
        newsPage: function ()
        {
            this.exitOnBack = false;

            $.mobile.loading("show", {
                text: "A carregar notícias...",
                textVisible: true,
                theme: "b"
            });

            var self = this;

            var collection = self.getCache('news', new NewsCollection());
            var page = new NewsListView({collection: collection});
            var serveCache = !Utils.isOnline();

            collection.fetch({
                cache: true,
                expires: Configs.cacheExpires.news, 
                serveCache: serveCache,
            }).done(function () {
                self.changePage( page, {'prependPageContent': new MenuView()} );
            }).fail(function () {

                self.changePage( page, {'prependPageContent': new MenuView()} );
                
                if (!self.popup)
                    self.popup = new PopupView({
                        'title': "Sem ligação", 
                        'content': "Não foi possível obter Notícias.</br> Verifique se possui uma ligação à rede activa. <a href='#' class='ui-btn ui-mini'>Voltar</a>"
                    }).render(); 

                self.popup.openPopup();
            });
        },

        // Learn Static Page method
        stepsStaticPage: function () 
        {
            this.exitOnBack = false;

            var page = new StaticView({template: "StepsTemplate", title: "Medidas em caso de sismo"});
            this.changePage( page, {'prependPageContent': new MenuView()} );
        },

        // Disclaimer Static Page method
        disclaimerStaticPage: function () 
        {
            this.exitOnBack = false;

            var page = new StaticView({template: "DisclaimerTemplate", title: "Disclaimer"});
            this.changePage( page, {'prependPageContent': new MenuView()} );
        },

    });

    // Returns the Router class
    return MobileRouter;
});