define([
    'jquery',
    'underscore',
    'backbone',
    'templates',
    'configs',
    'seismic/SeismicView',
    'leaflet',
], function(jquery, _, Backbone, Templates, Configs, SeismicView) {

    var SeismicListView = Backbone.View.extend({

        template: Templates.SeismicListTemplate,

        events: {
            'tap #searchBtn':                       "showSearchForm",
            'tap #seismic .ui-collapsible-heading': "centerMap"
        },

        initialize: function(options) 
        {
            this.map = null;

            _.bindAll(this, 'render', 'renderSeismicList', 'renderMap', 'centerMap');
            this.listenTo(this, 'inDOM', this.renderMap);
        },

        render: function() 
        {
            this.$el.html( this.template() );
            this.renderSeismicList();
            return this;
        },

        renderSeismicList: function()
        {
            var self = this;
            var container = document.createDocumentFragment();

            this.collection.forEach(function (seismic) {
                var seismicView = new SeismicView({model: seismic});
                container.appendChild( seismicView.render().el );
                self.addSubview(seismicView);
            });

            this.$("#seismic").empty().append(container);
        },

        renderMap: function () 
        {
            var self = this;
            var mapContainer = this.$("#mapSeismic");
            var markerStyle = Configs.seismicMarker.default;
            var layerSettings = {
                detectRetina: true, 
                maxZoom: 9, 
                minZoom: 2
            };

            self.layerGroup = new L.LayerGroup(); 

            var layer1 = L.tileLayer(Configs.leaflet.osmLayer, layerSettings);
            var layer2 = L.tileLayer(Configs.leaflet.esriWorldLayer, layerSettings);
            var layer3 = L.tileLayer(Configs.leaflet.esriOceanLayer, layerSettings);
            var layer4 = L.tileLayer(Configs.leaflet.omsRoadLayer, layerSettings);

            self.map = new L.Map(mapContainer[0], {
                attributionControl: false, 
                center: [36.7, -17.5],
                zoom: 4,
                layers: [layer1, layer2, layer3, layer4]
            });

            var baseMaps = {
                "OSM": layer1,
                "ESRI World": layer2,
                "ESRI Ocean": layer3,
                "OMS Road": layer4,
            };

            L.control.layers(baseMaps).addTo(self.map);

            self.map.whenReady(function() {
                var headerHeight = self.$('[data-role="header"]').outerHeight(true)-2;

                self.collection.forEach(function(model){
                    var circle = L.circleMarker([model.get('lat'), model.get('lon')], { 
                        radius: Math.round(model.get('magnitud'))+5, 
                        color: model.get('sensed') ? "black" : markerStyle.stroke, 
                        fillColor: markerStyle.fillColor, 
                        weight: 1, 
                        opacity: 1,
                        fillOpacity: markerStyle.fillOpacity,
                    });
                    // override the leaflet id with model id
                    circle._leaflet_id = model.cid;
                    circle.options.className = "seismic-marker-"+model.cid;
                    circle.addTo(self.layerGroup);
                    circle.on('click', function() {
                        var seismic = self.$('#seismic-'+model.cid);
                        $.mobile.silentScroll(seismic.offset().top-headerHeight);
                        seismic.find('[data-cid]').trigger('tap');
                        setTimeout(function() {
                            seismic.collapsible().collapsible("expand");
                        }, 100);
                    });
                });

                self.layerGroup.addTo(self.map);

                self.$('.loading').remove();
            });
        },

        centerMap: function (evt) 
        {
            var target = $(evt.currentTarget);

            var oldMarker = this.$('path.seismic-marker-selected');

            var model = this.collection.get(target.data('cid'));
            
            if (oldMarker.length) 
                this._unsetMarkerSelected( oldMarker, model.get('sensed') );

            if (target.hasClass("ui-collapsible-heading-collapsed")) {
                // center map 
                this.map.setView([model.get('lat'), model.get('lon')], 8, {animate: true});
                // remove all seismic-marker class and add it to cliked marker
                this._setMarkerSelected( this.$('path.seismic-marker-'+target.data('cid')), model.get('sensed') );
            }
        },

        _setMarkerSelected: function (marker, sensed) 
        {
            var markerStyle = Configs.seismicMarker.selected;
            marker.attr("class", marker.attr("class")+" seismic-marker-selected");
            marker.attr('fill', markerStyle.fillColor)
                  .attr('fill-opacity', markerStyle.fillOpacity)
                  .attr('stroke', sensed ? "black" : markerStyle.stroke);
        },

        _unsetMarkerSelected: function (marker, sensed)
        {
            var markerStyle = Configs.seismicMarker.default;
            marker.attr("class", marker.attr("class").replace("seismic-marker-selected", ""));
            marker.attr('fill', markerStyle.fillColor)
                  .attr('fill-opacity', markerStyle.fillOpacity)
                  .attr('stroke', sensed ? "black" : markerStyle.stroke);
        },

        showSearchForm: function (evt) 
        {
            evt.preventDefault();
            this.$("#search").toggleClass('hidden');
        }
    });

    return SeismicListView;
});
